/* ---------------------------------------------------------------
Création le 10/8/2019
Mise à jour le 26/2/2021
-> mqttClient remplacé par PubSubClient
Sources : Hackable Magazine n°26
Auteurs : Denis Bodor
Titre : Client MQTT
Fonction : template
Description : Publication d'une valeur incrémentée et abonnement
à une commande on/off destinée à commander la LED de la carte
Cibles : Carte mkr1010 MKR MKR 1010
Fichier : ARD_MKR1010_MQTT_V2.ino
----------------------------------------------------------------*/
// Bibliothèques
// ---------------------------------------------------------------------------------
// MQTT : implémentation d'un client MQTT
#include <PubSubClient.h>
// Connexion au wifi
#include <WiFiNINA.h> // si MKR1000 changer par #include <WiFi101.h>
// Paramètres de connexion au broker
#include "parametres.h"
// ---------------------------------------------------------------------------------
// Paramètres WIFI A RENSEIGNER dans parametres.h
// NE PAS MODIFIER le code ci-dessous
// ---------------------------------------------------------------------------------
const char *ssid = SECRET_SSID;     // SSID du réseau.
const char *password = SECRET_PASS; // Mot de passe du réseau.
// Ce nom est repris dans la construction des topics.
// ---------------------------------------------------------------------------------
// Paramètres pour la connexion au Broker A RENSEIGNER dans parametres.h
// NE PAS MODIFIER le code ci-dessous
// ---------------------------------------------------------------------------------
// IP de la machine sur laquelle est installé le broker et port
const char *mqtt_server = BROKER;
const uint16_t mqtt_PORT = MQTTPORT; // Port TCP sur lequel le broker écoute
// (par défaut pour brokers MQTT)
const char *myhostname = MQTTCLIENTID; // Nom donné au client MQTT.
const uint8_t qos = QoS;               // Qualité de service
// ---------------------------------------------------------------------------------
// Les topics (in = Abonnement / out = publication) SONT A DEFINIR dans parametres.h
// #### Constantes ci-dessous A ADAPTER en fonction du fichier parametres.h ####
// ---------------------------------------------------------------------------------
const char *inTopic = TOPIC_IN;
const char *outTopic = TOPIC_OUT;
// ---------------------------------------------------------------------------------
// Objet pour la connexion au réseau wifi
WiFiClient mkr1010Client;
// Objet pour la connexion au broker MQTT (Publisher/Subscriber)
PubSubClient mqttClient(mkr1010Client);
// Intervalle de temps séparant la publication du (des) topic()s
const long interval = INTERVAL; // A RENSEIGNER dans parametres.h
// Permet de calculer l'intervalle de temps depuis la dernière
// publication du (des) topic(s)
unsigned long previousMillis = 0;
// Valeur à publier (pour tester la connexion)
uint32_t compteur = 0;
// ---------------------------------------------------------------------------------
// Fonctions
// ---------------------------------------------------------------------------------
// Connexion au Wifi
// ---------------------------------------------------------------------------------
void setup_wifi()
{
  /* 
  * TODO Mode station : déclaration à voir ????
  */
  Serial.println();
  Serial.print("Tentative de connexion à ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED)
  {
    delay(5000);
    Serial.print(".");
  }
  // Affichage
  Serial.println("");
  Serial.println("Connexion au Wifi ok");
  Serial.print("Adresse IP: ");
  Serial.println(WiFi.localIP());
  /* 
  * TODO Configuration de mDNS
  */
}
// ---------------------------------------------------------------------------------
// Traitement des messages issus du broker
// #### A ADAPTER en fonction du ou des messages reçus ####
// Ici, on commande la led BUILTIN de la carte
// ---------------------------------------------------------------------------------
// Fonction de rappel (appelée automatiquement lors de la réception d'un message)
// topic : chaîne de caractères décrivant le topic
// payload : message associé au topic
// length : nombre d'octets du message
void callback(char *topic, byte *payload, unsigned int length)
{
  Serial.print("Message [");
  Serial.print(topic);
  Serial.print("] ");
  // Affichage du message
  for (int i = 0; i < length; i++)
  {
    Serial.print((char)payload[i]);
  }
  Serial.println("");
  // '1' est-il le premier caractère du message ?
  if ((char)payload[0] == '0')
  {
    // Oui led=on
    digitalWrite(LED_BUILTIN, LOW);
  }
  else
  {
    // Non led=off
    digitalWrite(LED_BUILTIN, HIGH);
  }
}
// ---------------------------------------------------------------------------------
// Maintien de la connexion au broker MQTT et abonnement au(x) topic(s)
// #### A ADAPTER en fonction du nombre d'abonnements  ####
// ---------------------------------------------------------------------------------
void reconnect()
{
  // Connecté au broker ?
  while (!mqttClient.connected())
  {
    // Non, on se connecte
    if (!mqttClient.connect(myhostname))
    {
      Serial.print("Erreur de connexion MQTT, rc=");
      Serial.println(mqttClient.state());
      delay(5000);
      continue;
    }
    Serial.println("Connexion serveur MQTT ok");
    // Connecté ! On s'abonne au topic (ici "ctrl_led")
    mqttClient.subscribe(inTopic, qos);
    Serial.print("Abonnement au topic ");
    Serial.println(inTopic);
  }
}

// ---------------------------------------------------------------------------------
// Programme principal : initialisation
// ---------------------------------------------------------------------------------
// Initialisations
void setup()
{
  // Configuration de la broche connectée à la LED de la carte
  pinMode(LED_BUILTIN, OUTPUT);
  // Configuration du moniteur série
  /*  
  Serial.begin(115200);
  while (!Serial) // Mode debug
    ;             // A supprimer si la carte n'est plus connectée au PC
*/
  // Connexion au Wifi
  setup_wifi();
  // Configuration du broker
  // MQTT_PORT = 1883 (port TCP par défaut des brokers MQTT)
  mqttClient.setServer(mqtt_server, mqtt_PORT);
  // Activation de la fonction callback
  mqttClient.setCallback(callback);
}
// ---------------------------------------------------------------------------------
// Programme principal : boucle infinie
// ---------------------------------------------------------------------------------
void loop()
{
  // Tableau(x) pour la mise en forme de la (ou des) valeur(s) publiée(s)
  char msgTest[16];

  // Tableau(x) pour le(s) topic(s) construit(s) avec le nom de l'hôte
  char topicTest[64];

  // Sommes-nous connectés au broker MQTT ?
  if (!mqttClient.connected())
  {
    // Non alors
    reconnect(); // maintenir la connexion et s'abonner à un ou plusieurs topics
  }
  // Oui, maintien de la connexion avec le broker
  // Interrogation du broker : ne doit pas être bloquée !!! pour traiter les messages reçus
  mqttClient.loop(); // Déclencle la fonction callback si message reçu

  // Temporisation "non blocante" avant l'envoi d'un ou plusieurs topics
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= interval)
  {
    // On mémorise la "date" à laquelle le ou les messages ont été publiés
    previousMillis = currentMillis;
    compteur++; // Incrémentation de la valeur à publier (test)
                // Construction du ou des message(s)
    sprintf(msgTest, "%hu", compteur);
    // Construction du (des) topic(s) avec le nom d'hôte
    // Exemple "<myhostname>/<valeur>" #### A ADAPTER ####
    sprintf(topicTest, "%s/%s", myhostname, outTopic);
    // Publication du (des) message(s) sur le(s) topic(s)
    mqttClient.publish(topicTest, msgTest);
  }
}
// ---------------------------------------------------------------------------------